package fasthttp

import (
	"fmt"
	"strings"

	logging "git.loafle.net/commons/logging-go"
	"git.loafle.net/commons/server-go"
	"git.loafle.net/commons/server-go/web"

	"github.com/valyala/fasthttp"
)

type ServerHandler interface {
	web.ServerHandler

	OnError(serverCtx server.ServerCtx, ctx *fasthttp.RequestCtx, status int, reason error)

	RegisterServlet(path string, servlet Servlet)
	Servlet(serverCtx server.ServerCtx, ctx *fasthttp.RequestCtx) Servlet

	CheckOrigin(ctx *fasthttp.RequestCtx) bool
}

type ServerHandlers struct {
	web.ServerHandlers

	NotFoundServelt Servlet

	// path = context only.
	// ex) /auth => /auth, /auth/member => /auth
	servlets map[string]Servlet
}

func (sh *ServerHandlers) Init(serverCtx server.ServerCtx) error {
	if err := sh.ServerHandlers.Init(serverCtx); nil != err {
		return err
	}

	if nil != sh.servlets {
		for _, servlet := range sh.servlets {
			if err := servlet.Init(serverCtx); nil != err {
				return err
			}
		}
	}

	return nil
}

func (sh *ServerHandlers) Destroy(serverCtx server.ServerCtx) {
	if nil != sh.servlets {
		for _, servlet := range sh.servlets {
			servlet.Destroy(serverCtx)
		}
	}

	sh.ServerHandlers.Destroy(serverCtx)
}

func (sh *ServerHandlers) OnError(serverCtx server.ServerCtx, ctx *fasthttp.RequestCtx, status int, reason error) {
	ctx.Error(reason.Error(), status)
}

func (sh *ServerHandlers) RegisterServlet(contextPath string, servlet Servlet) {
	if nil == sh.servlets {
		sh.servlets = make(map[string]Servlet)
	}
	sh.servlets[contextPath] = servlet
}

func (sh *ServerHandlers) Servlet(serverCtx server.ServerCtx, ctx *fasthttp.RequestCtx) Servlet {
	contextPath, err := getContextPath(string(ctx.Path()))
	if nil != err {
		logging.Logger().Warnf("%v", err)
		return sh.NotFoundServelt
	}

	var servlet Servlet
	if servlet = sh.servlets[contextPath]; nil == servlet {
		return sh.NotFoundServelt
	}

	return servlet
}

func (sh *ServerHandlers) CheckOrigin(ctx *fasthttp.RequestCtx) bool {
	return true
}

func (sh *ServerHandlers) Validate() error {
	if err := sh.ServerHandlers.Validate(); nil != err {
		return err
	}

	return nil
}

func getContextPath(path string) (string, error) {
	p := strings.TrimSpace(path)

	if !strings.HasPrefix(p, "/") {
		return "", fmt.Errorf("The path[%s] must started /", path)
	}
	p = p[1:]

	if strings.HasSuffix(p, "/") {
		cpl := len(p) - 1
		p = p[:cpl]
	}

	components := strings.Split(p, "/")
	if 0 == len(components) {
		return "", fmt.Errorf("The path[%s] is not invalid", path)
	}

	return fmt.Sprintf("/%s", components[0]), nil
}
